@@ -2,8 +2,8 @@ from __future__ import print_function
 
 class Aggregator(object):
     def __init__(self):
-        self.viewers={}
-        self.symbols=[]
+        self.viewers = {}
+        self.symbols = []
 
     def add_symbols(self, symbols):
         self.symbols.extend(symbols)
@@ -13,7 +13,7 @@ class Aggregator(object):
 
     def view(self, viewer, symbols):
         print("aggregator gets a new viewer, for symbols:", symbols)
-        self.viewers[viewer]=symbols
+        self.viewers[viewer] = symbols
 
     def quotes(self, market, stockquotes):
         for symbol, value in stockquotes.items():
@@ -23,9 +23,11 @@ class Aggregator(object):
 
 
 def main(stockmarkets):
-    aggregator=Aggregator()
+    aggregator = Aggregator()
     for market in stockmarkets:
         aggregator.add_symbols(market.symbols())
         market.listener(aggregator)
+    if not aggregator.available_symbols():
+        raise ValueError("no symbols found!")
     print("aggregated symbols:", aggregator.available_symbols())
     return aggregator
