@@ -6,8 +6,13 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 
 import sys, zlib, logging
 import traceback, linecache
+try:
+    import copyreg
+except ImportError:
+    import copy_reg as copyreg
 import Pyro4
 import Pyro4.errors
+import Pyro4.message
 
 log=logging.getLogger("Pyro4.util")
 
@@ -120,12 +125,12 @@ all_exceptions = {}
 if sys.version_info < (3, 0):
     import exceptions
     for name, t in vars(exceptions).items():
-        if type(t) is type and issubclass(t, Exception):
+        if type(t) is type and issubclass(t, BaseException):
             all_exceptions[name] = t
 else:
     import builtins
     for name, t in vars(builtins).items():
-        if type(t) is type and issubclass(t, Exception):
+        if type(t) is type and issubclass(t, BaseException):
             all_exceptions[name] = t
 for name, t in vars(Pyro4.errors).items():
     if type(t) is type and issubclass(t, Pyro4.errors.PyroError):
@@ -181,6 +186,10 @@ class SerializerBase(object):
             return compressed, True
         return data, False
 
+    @classmethod
+    def register_type_replacement(cls, object_type, replacement_function):
+        raise NotImplementedError("implement in subclass")
+
     @classmethod
     def register_class_to_dict(cls, clazz, converter):
         cls.__custom_class_to_dict_registry[clazz] = converter
@@ -200,7 +209,7 @@ class SerializerBase(object):
             raise Pyro4.errors.ProtocolError("couldn't serialize sequence " + str(obj.__class__) + ", one of its elements is unserializable")
         if hasattr(obj, "_pyroDaemon"):
             obj._pyroDaemon = None
-        if isinstance(obj, Exception):
+        if isinstance(obj, BaseException):
             # special case for exceptions
             value={"args": obj.args}
             value["__class__"] = obj.__class__.__module__ + "." + obj.__class__.__name__
@@ -265,13 +274,16 @@ class SerializerBase(object):
             errortype = getattr(Pyro4.errors, classname.split('.', 2)[2])
             if issubclass(errortype, Pyro4.errors.PyroError):
                 return SerializerBase.make_exception(errortype, data)
+        elif classname == "Pyro4.futures._ExceptionWrapper":
+            ex = SerializerBase.dict_to_class(data["exception"])
+            return Pyro4.futures._ExceptionWrapper(ex)
         elif classname.startswith("builtins."):
             exceptiontype = getattr(builtins, classname.split('.', 1)[1])
-            if issubclass(exceptiontype, Exception):
+            if issubclass(exceptiontype, BaseException):
                 return SerializerBase.make_exception(exceptiontype, data)
         elif classname.startswith("exceptions."):
             exceptiontype = getattr(exceptions, classname.split('.', 1)[1])
-            if issubclass(exceptiontype, Exception):
+            if issubclass(exceptiontype, BaseException):
                 return SerializerBase.make_exception(exceptiontype, data)
         elif classname in all_exceptions:
             return SerializerBase.make_exception(all_exceptions[classname], data)
@@ -321,6 +333,8 @@ class PickleSerializer(SerializerBase):
     A (de)serializer that wraps the Pickle serialization protocol.
     It can optionally compress the serialized data, and is thread safe.
     """
+    serializer_id = Pyro4.message.SERIALIZER_PICKLE
+
     def dumpsCall(self, obj, method, vargs, kwargs):
         return pickle.dumps((obj, method, vargs, kwargs), pickle.HIGHEST_PROTOCOL)
 
@@ -333,9 +347,20 @@ class PickleSerializer(SerializerBase):
     def loads(self, data):
         return pickle.loads(data)
 
+    @classmethod
+    def register_type_replacement(cls, object_type, replacement_function):
+        def copyreg_function(obj):
+            return replacement_function(obj).__reduce__()
+        try:
+            copyreg.pickle(object_type, copyreg_function)
+        except TypeError:
+            pass
+
 
 class MarshalSerializer(SerializerBase):
     """(de)serializer that wraps the marshal serialization protocol."""
+    serializer_id = Pyro4.message.SERIALIZER_MARSHAL
+
     def dumpsCall(self, obj, method, vargs, kwargs):
         return marshal.dumps((obj, method, vargs, kwargs))
 
@@ -354,9 +379,15 @@ class MarshalSerializer(SerializerBase):
     def loads(self, data):
         return self.recreate_classes(marshal.loads(data))
 
+    @classmethod
+    def register_type_replacement(cls, object_type, replacement_function):
+        pass    # marshal serializer doesn't support per-type hooks
+
 
 class SerpentSerializer(SerializerBase):
     """(de)serializer that wraps the serpent serialization protocol."""
+    serializer_id = Pyro4.message.SERIALIZER_SERPENT
+
     def dumpsCall(self, obj, method, vargs, kwargs):
         return serpent.dumps((obj, method, vargs, kwargs))
 
@@ -372,9 +403,22 @@ class SerpentSerializer(SerializerBase):
     def loads(self, data):
         return self.recreate_classes(serpent.loads(data))
 
+    @classmethod
+    def register_type_replacement(cls, object_type, replacement_function):
+        def custom_serializer(object, serpent_serializer, outputstream, indentlevel):
+            replaced = replacement_function(object)
+            if replaced is object:
+                serpent_serializer.ser_default_class(replaced, outputstream, indentlevel)
+            else:
+                serpent_serializer._serialize(replaced, outputstream, indentlevel)
+        serpent.register_class(object_type, custom_serializer)
+
 
 class JsonSerializer(SerializerBase):
     """(de)serializer that wraps the json serialization protocol."""
+    serializer_id = Pyro4.message.SERIALIZER_JSON
+
+    __type_replacements = {}
     def dumpsCall(self, obj, method, vargs, kwargs):
         data = {"object": obj, "method": method, "params": vargs, "kwargs": kwargs}
         data = json.dumps(data, ensure_ascii=False, default=self.default)
@@ -392,37 +436,66 @@ class JsonSerializer(SerializerBase):
         data=data.decode("utf-8")
         return self.recreate_classes(json.loads(data))
     def default(self, obj):
+        replacer = self.__type_replacements.get(type(obj), None)
+        if replacer:
+            obj = replacer(obj)
         return self.class_to_dict(obj)
+    @classmethod
+    def register_type_replacement(cls, object_type, replacement_function):
+        cls.__type_replacements[object_type] = replacement_function
+
 
 """The various serializers that are supported"""
 _serializers = {}
-def get_serializer():
+_serializers_by_id = {}
+def get_serializer(name):
     try:
-        return _serializers[Pyro4.config.SERIALIZER]
+        return _serializers[name]
     except KeyError:
-        raise Pyro4.errors.ProtocolError("configured serializer '%s' is unknown or not available" % Pyro4.config.SERIALIZER)
+        raise Pyro4.errors.ProtocolError("serializer '%s' is unknown or not available" % name)
+
+def get_serializer_by_id(sid):
+    try:
+        return _serializers_by_id[sid]
+    except KeyError:
+        raise Pyro4.errors.ProtocolError("no serializer available for id %d" % sid)
 
 # determine the serializers that are supported
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
-if pickle.HIGHEST_PROTOCOL<2:
-    raise RuntimeError("pickle serializer needs to support protocol 2 or higher")
-_serializers["pickle"] = PickleSerializer()
+assert pickle.HIGHEST_PROTOCOL>=2, "pickle needs to support protocol 2 or higher"
+_ser = PickleSerializer()
+_serializers["pickle"] = _ser
+_serializers_by_id[_ser.serializer_id] = _ser
 import marshal
-_serializers["marshal"] = MarshalSerializer()
+_ser = MarshalSerializer()
+_serializers["marshal"] = _ser
+_serializers_by_id[_ser.serializer_id] = _ser
 try:
     import json
-    _serializers["json"] = JsonSerializer()
+    _ser = JsonSerializer()
+    _serializers["json"] = _ser
+    _serializers_by_id[_ser.serializer_id] = _ser
 except ImportError:
     pass
 try:
     import serpent
-    _serializers["serpent"] = SerpentSerializer()
+    if '-' in serpent.__version__:
+        ver = serpent.__version__.split('-', 1)[0]
+    else:
+        ver = serpent.__version__
+    ver = tuple(map(int, ver.split(".")))
+    if ver<(1,3):
+        raise RuntimeError("requires serpent 1.3 or better")
+    _ser = SerpentSerializer()
+    _serializers["serpent"] = _ser
+    _serializers_by_id[_ser.serializer_id] = _ser
 except ImportError:
     #warnings.warn("serpent serializer not available", RuntimeWarning)
     pass
+del _ser
 
 
 def resolveDottedAttribute(obj, attr, allowDotted):
@@ -451,8 +524,11 @@ def excepthook(ex_type, ex_value, ex_tb):
 
 
 def fixIronPythonExceptionForPickle(exceptionObject, addAttributes):
-    """function to hack around a bug in IronPython where it doesn't pickle
-    exception attributes. We piggyback them into the exception's args."""
+    """
+    Function to hack around a bug in IronPython where it doesn't pickle
+    exception attributes. We piggyback them into the exception's args.
+    Bug report is at http://ironpython.codeplex.com/workitem/30805
+    """
     if hasattr(exceptionObject, "args"):
         if addAttributes:
             # piggyback the attributes on the exception args instead.
