@@ -11,7 +11,8 @@ import threading
 import pickle
 
 
-__all__=["tobytes", "unicode", "unichr", "StringIO", "next", "AtomicCounter", "NonserializableError", "MyThing2"]
+__all__=["tobytes", "unicode", "unichr", "basestring", "StringIO", "next", "AtomicCounter",
+        "NonserializableError", "MyThing2", "unittest" ]
 
 if sys.version_info<(3,0):
     from StringIO import StringIO
@@ -19,12 +20,14 @@ if sys.version_info<(3,0):
         return string
     unicode=unicode
     unichr=unichr
+    basestring=basestring
 else:
     from io import StringIO
     def tobytes(string, encoding="iso-8859-1"):
         return bytes(string,encoding)
     unicode=str
     unichr=chr
+    basestring=str
 
 
 if sys.version_info<(2,6):
@@ -34,6 +37,13 @@ else:
     next=next
 
 
+if (sys.version_info >= (2, 7) and sys.version_info < (3, 0)) or \
+        (sys.version_info >= (3, 1)):
+    import unittest
+else:
+    import unittest2 as unittest
+
+
 class AtomicCounter(object):
     def __init__(self):
         self.lock = threading.Lock()
@@ -54,4 +64,7 @@ class NonserializableError(Exception):
 
 
 class MyThing2(object):
-    pass
+    def __init__(self, name="?"):
+        self.name = name
+
+
